<?php

namespace Drupal\monarch_entity_updater\Commands;

use Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface;
use Drush\Commands\DrushCommands;
use Drush\Utils\StringUtils;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class MonarchEntityUpdaterCommands extends DrushCommands {

  /**
   * Rudimentary translation system, akin to Drupal's t() function.
   *
   * @param string $string
   *   String to process, possibly with replacement item.
   * @param array $args
   *   An associative array of replacement items.
   *
   * @return mixed
   *   The processed string.
   */
  protected function dt(string $string, array $args = []) {
    return StringUtils::interpolate($string, $args);
  }

  /**
   * Command description here.
   *
   * @param string $entity_type_id
   *   The entity type to update.
   * @param array $options
   *   An associative array of options.
   *
   * @usage monarch_entity_updater:do_entity_updates node
   *   Updates node entity definitions.
   *
   * @command monarch_entity_updater:do_entity_updates
   * @aliases eu
   */
  public function doEntityUpdates(string $entity_type_id = NULL, array $options = NULL) {
    if (empty($entity_type_id)) {
      $change_list = \Drupal::service('monarch_entity_updater')->getChangeList() ?? [];

      if (empty($change_list)) {
        $this->output()->writeln($this->dt('No changes needed for @entitytype.', [
          '@entitytype' => $entity_type_id,
        ]));
        return;
      }

      $this->output()->writeln($this->dt('Changes needed:'));
      foreach ($change_list as $change_entity_type_id => $categories) {
        foreach (($categories ?? []) as $category => $changes) {
          if (is_array($changes)) {
            foreach ($changes as $change_name => $change_type) {
              if ($change_type === EntityDefinitionUpdateManagerInterface::DEFINITION_CREATED) {
                $this->output()->writeln(" [$category] $change_entity_type_id.$change_name: Created");
              }
              elseif ($change_type === EntityDefinitionUpdateManagerInterface::DEFINITION_UPDATED) {
                $this->output()->writeln(" [$category] $change_entity_type_id.$change_name: Updated");
              }
              elseif ($change_type === EntityDefinitionUpdateManagerInterface::DEFINITION_DELETED) {
                $this->output()->writeln(" [$category] $change_entity_type_id.$change_name: Deleted");
              }
              else {
                $this->output()->writeln(" [$category] $change_entity_type_id.$change_name: Unknown");
              }
            }
          }
          elseif (is_numeric($changes)) {
            $change_type = $changes;

            if ($change_type === EntityDefinitionUpdateManagerInterface::DEFINITION_CREATED) {
              $this->output()->writeln(" [$category] $change_entity_type_id: Created");
            }
            elseif ($change_type === EntityDefinitionUpdateManagerInterface::DEFINITION_UPDATED) {
              $this->output()->writeln(" [$category] $change_entity_type_id: Updated");
            }
            elseif ($change_type === EntityDefinitionUpdateManagerInterface::DEFINITION_DELETED) {
              $this->output()->writeln(" [$category] $change_entity_type_id: Deleted");
            }
            else {
              $this->output()->writeln(" [$category] $change_entity_type_id: Unknown");
            }
          }
        }
      }

      return;
    }

    $changes = \Drupal::service('monarch_entity_updater')->getChangeList($entity_type_id);

    if (!empty($changes)) {
      \Drupal::service('monarch_entity_updater')->doEntityUpdates($entity_type_id);
      $this->output()->writeln($this->dt('Updated @entitytype definitions.', [
        '@entitytype' => $entity_type_id,
      ]));
    }
    else {
      $this->output()->writeln($this->dt('No updates available for @entitytype.', [
        '@entitytype' => $entity_type_id,
      ]));
    }
  }

}
