<?php

namespace Drupal\monarch_entity_updater;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface;
use Drupal\Core\Entity\EntityTypeListenerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldStorageDefinitionListenerInterface;

/**
 * Service description.
 */
class MonarchEntityUpdater {

  /**
   * The entity type listener service.
   *
   * @var \Drupal\Core\Entity\EntityTypeListenerInterface
   */
  protected $entityTypeListener;

  /**
   * The field storage definition listener service.
   *
   * @var \Drupal\Core\Field\FieldStorageDefinitionListenerInterface
   */
  protected $fieldStorageDefinitionListener;

  /**
   * The entity last installed schema repository.
   *
   * @var \Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface
   */
  protected $entityLastInstalledSchemaRepository;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Returns the entity definition update manager.
   *
   * @var \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface
   */
  protected $entityDefinitionUpdateManager;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs a MonarchEntityUpdater object.
   *
   * @param \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface $entity_definition_update_manager_interface
   *   The entity definition update manager service.
   * @param \Drupal\Core\Entity\EntityTypeListenerInterface $entity_type_listener
   *   The entity type listener service.
   * @param \Drupal\Core\Field\FieldStorageDefinitionListenerInterface $field_storage_definition_listener
   *   The field storage definition listener service.
   * @param \Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface $entity_last_installed_schema_repository
   *   The entity last installed schema repository.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(EntityDefinitionUpdateManagerInterface $entity_definition_update_manager_interface, EntityTypeListenerInterface $entity_type_listener, FieldStorageDefinitionListenerInterface $field_storage_definition_listener, EntityLastInstalledSchemaRepositoryInterface $entity_last_installed_schema_repository, EntityFieldManagerInterface $entity_field_manager, EntityTypeManagerInterface $entity_type_manager, Connection $connection) {
    $this->entityDefinitionUpdateManager = $entity_definition_update_manager_interface;
    $this->entityTypeListener = $entity_type_listener;
    $this->fieldStorageDefinitionListener = $field_storage_definition_listener;
    $this->entityLastInstalledSchemaRepository = $entity_last_installed_schema_repository;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->connection = $connection;
  }

  /**
   * Get the change list.
   *
   * @param string $entity_type_id
   *   The entity type id to check for changes.
   * @param string $change_type
   *   The type of change to check for.
   *
   * @return array|null
   *   The change list, or null.
   */
  public function getChangeList(string $entity_type_id = NULL, string $change_type = NULL) : ?array {
    $ret = $this->entityDefinitionUpdateManager->getChangeList();

    if (!is_null($entity_type_id)) {
      if (!is_null($change_type)) {
        return $ret[$entity_type_id][$change_type] ?? NULL;
      }

      return $ret[$entity_type_id] ?? NULL;
    }

    return $ret;
  }

  /**
   * Process standard field updates for an entity type.
   *
   * @param string $entity_type_id
   *   The entity type id to process updates for.
   *
   * @return void
   *   No return value.
   */
  public function doEntityUpdates(string $entity_type_id) : void {
    if (!empty($this->getChangeList($entity_type_id, 'entity_type'))) {
      throw new \Error('Entity changes are pending. Ensure that all modules are installed properly and updated.');
    }

    if (!empty($field_storage_definitions = $this->getChangeList($entity_type_id, 'field_storage_definitions'))) {
      foreach ($field_storage_definitions as $field_name => $status) {
        switch ($status) {
          case EntityDefinitionUpdateManagerInterface::DEFINITION_CREATED:
            $field_storage_definition = $this->entityFieldManager->getFieldStorageDefinitions($entity_type_id)[$field_name];
            $this->fieldStorageDefinitionListener->onFieldStorageDefinitionCreate($field_storage_definition);
            break;

          case EntityDefinitionUpdateManagerInterface::DEFINITION_UPDATED:
            $field_storage_definition = $this->entityFieldManager->getFieldStorageDefinitions($entity_type_id)[$field_name];
            $old_field_storage_definition = $this->entityLastInstalledSchemaRepository->getLastInstalledFieldStorageDefinitions($entity_type_id)[$field_name];
            $this->fieldStorageDefinitionListener->onFieldStorageDefinitionUpdate($field_storage_definition, $old_field_storage_definition);
            break;

          case EntityDefinitionUpdateManagerInterface::DEFINITION_DELETED:
            $field_storage_definition = $this->entityLastInstalledSchemaRepository->getLastInstalledFieldStorageDefinitions($entity_type_id)[$field_name];
            $this->fieldStorageDefinitionListener->onFieldStorageDefinitionDelete($field_storage_definition);
            break;

          default:
            break;
        }
      }
    }
  }

}
